self.__BUILD_MANIFEST=function(s,c,e,a,t,u,n,f,r,i,j,b,k,o,h,d){return{__rewrites:{afterFiles:[],beforeFiles:[],fallback:[]},"/":["static/chunks/pages/index-444f1804401f04ea.js"],"/_error":["static/chunks/pages/_error-c66a4e8afc46f17b.js"],"/clusters":["static/chunks/pages/clusters-956ad430075efee8.js"],"/clusters/[cluster]":[s,c,e,a,t,r,j,b,u,n,f,k,i,o,h,d,"static/chunks/1871-7e17c195296e2ea9.js","static/chunks/pages/clusters/[cluster]-9e7df5fc761c95a7.js"],"/clusters/[cluster]/[job]":[s,c,e,a,t,u,n,f,"static/chunks/pages/clusters/[cluster]/[job]-fa63e8b1d203f298.js"],"/config":["static/chunks/pages/config-8620d099cbef8608.js"],"/infra":["static/chunks/pages/infra-0fbdc9072f19fbe2.js"],"/infra/[context]":["static/chunks/pages/infra/[context]-9cfd875eecb6eaf5.js"],"/jobs":["static/chunks/pages/jobs-6393a9edc7322b54.js"],"/jobs/[job]":[s,c,e,a,t,r,b,u,n,f,i,"static/chunks/pages/jobs/[job]-6c5af4c86e6ab3d3.js"],"/users":["static/chunks/pages/users-34d6bb10c3b3ee3d.js"],"/volumes":["static/chunks/pages/volumes-225c8dae0634eb7f.js"],"/workspace/new":["static/chunks/pages/workspace/new-92f741084a89e27b.js"],"/workspaces":["static/chunks/pages/workspaces-e4cb7e97d37e93ad.js"],"/workspaces/[name]":[s,c,e,a,t,r,j,u,n,f,k,i,o,h,d,"static/chunks/1141-3f10a5a9f697c630.js","static/chunks/pages/workspaces/[name]-4d41c9023287f59a.js"],sortedPages:["/","/_app","/_error","/clusters","/clusters/[cluster]","/clusters/[cluster]/[job]","/config","/infra","/infra/[context]","/jobs","/jobs/[job]","/users","/volumes","/workspace/new","/workspaces","/workspaces/[name]"]}}("static/chunks/616-3d59f75e2ccf9321.js","static/chunks/5230-f3bb2663e442e86c.js","static/chunks/5739-d67458fcb1386c92.js","static/chunks/1664-22b00e32c9ff96a4.js","static/chunks/7411-b15471acd2cba716.js","static/chunks/6989-983d3ae7a874de98.js","static/chunks/3698-7874720877646365.js","static/chunks/6135-d0e285ac5f3f2485.js","static/chunks/1272-1ef0bf0237faccdb.js","static/chunks/8969-9a8cca241b30db83.js","static/chunks/1559-6c00e20454194859.js","static/chunks/6212-7bd06f60ba693125.js","static/chunks/6990-08b2a1cae076a943.js","static/chunks/1043-928582d4860fef92.js","static/chunks/6601-234b1cf963c7280b.js","static/chunks/938-40d15b6261ec8dc1.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();