(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5963],{35113:function(e,n,r){(window.__NEXT_P=window.__NEXT_P||[]).push(["/workspace/new",function(){return r(4250)}])},4250:function(e,n,r){"use strict";r.r(n),r.d(n,{default:function(){return u}});var t=r(85893);r(67294);var s=r(9008),c=r.n(s),i=r(5152);let o=r.n(i)()(()=>Promise.all([r.e(5230),r.e(1664),r.e(6990),r.e(3785)]).then(r.bind(r,33785)).then(e=>e.NewWorkspace),{loadableGenerated:{webpack:()=>[33785]},ssr:!1});function u(){return(0,t.jsxs)(t.Fragment,{children:[(0,t.jsx)(c(),{children:(0,t.jsx)("title",{children:"New Workspace | SkyPilot Dashboard"})}),(0,t.jsx)(o,{})]})}},9008:function(e,n,r){e.exports=r(37219)}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=35113)}),_N_E=e.O()}]);